import 'reflect-metadata';
import * as Promise from 'bluebird';
import ObjectFactory from './object-factory';
import { default as ObjectWrapper } from './object-wrapper';
export { ObjectWrapper, ObjectFactory };
export declare namespace Di {
    interface DisposerFactory<T> {
        (resource: T): Promise.Disposer<T>;
    }
    interface ScopeResource {
        constructor: new (...args: any[]) => any;
        disposerFactory: DisposerFactory<any>;
    }
    class Container {
        private kernel;
        private scopeResources;
        private boundConstants;
        constructor();
        bindTransientClass(aClass: new (...args: any[]) => any, bClass?: new (...args: any[]) => any): Container;
        bindScopeResource<T>(aClass: new (...args: any[]) => T, disposerFactory: DisposerFactory<T>): Container;
        private decorateInjectable;
        getConstantValue<T>(identifier: InjectionIdentifier<any>): T;
        getConstantIdentifierList(): InjectionIdentifier<any>[];
        bindConstant(identifier: InjectionIdentifier<any>, value: any): Container;
        bindObjectWrapper(aClass: new (...args: any[]) => any): Container;
        scope(): Scope;
    }
    type InjectionIdentifier<T> = string | (new (...args: any[]) => T);
    class Scope {
        private scopeResources;
        private kernel;
        private objToBindScopeContext;
        private injections;
        private disposers;
        constructor(kernel: any, scopeResources: ScopeResource[]);
        context(contextToBind: {
            [name: string]: any;
        }): Scope;
        inject(...args: InjectionIdentifier<any>[]): Scope;
        run<R>(task: (...args: any[]) => Promise<R> | R): Promise<R>;
        private bindScopeContext;
        private bindScopeResources;
        private injectScopeParameters;
    }
    class ScopeContext {
        private context;
        setOnce(name: string, value: any): ScopeContext;
        get<T>(name: string): T;
    }
    function inject(target: any, key?: string, index?: number): any;
    function scope(target: any, name: string, descriptor: PropertyDescriptor): any;
    function bindTransientClass(aClass: new (...args: any[]) => any): void;
    var container: Container;
}
